var assert = require("assert");
var tp = require("./");

var GrandParent = new tp.Class({
  prototype: {
    field: 5,
  },
});



var Parent = new tp.Class({
  parent: GrandParent,
  constructor: function (field) {
    this.field = field;
  },
});


var Child = new tp.Class({
  parent: Parent,
  prototype: {
    constructor: function () {
      this.__super__(3);
    },
    sayHello: function () {
      return Child.MESSAGE + this.field;
    },
  },
  statics: {
    MESSAGE: "My field is ",
  },
});

assert(!(new Parent instanceof Child));
assert(new Child instanceof Child);
assert(new Child instanceof Parent);
assert(new Child instanceof GrandParent);
assert(new Child().sayHello() == "My field is 3");

var MyEnum = new tp.Enum({
  FOO: {
    toString: function () {
      return "Foo";
    },
  },
  BAR: {
    toString: function () {
      return "Bar";
    },
  },
});

assert(MyEnum.validate(MyEnum.FOO));
assert(!MyEnum.validate({}));
assert(MyEnum.eq(MyEnum.FOO, MyEnum.FOO));
assert(!MyEnum.eq(MyEnum.BAR, MyEnum.FOO));
assert.deepEqual(MyEnum.keys(), ["FOO", "BAR"]);


var ToString = new tp.Trait("ToString", {
  toString: null,
});

var Encodable = new tp.Trait("Encodable", {
  encode: null,
  asEncodedArray: function () {
    return this.encode().split().map(function (val) {
      return val.charCodeAt(0);
    });
  },
}).fromParent(ToString);


Encodable.implementFor(Child.prototype, {
  encode: function () {
    return String.fromCharCode(this.field);
  },
});


assert(Encodable.validate(new Child));
assert(!Encodable.validate(new Parent));

console.log("Success")
